import { setConfig, getConfig } from './config.js'
import db from './db.js'
import fs from 'fs'
import { createServer } from 'http'
import path from 'path'
import querystring from 'querystring'

setConfig()
const Config = getConfig()

const HEAD = fs.readFileSync(path.join(Config.WWW, 'head.html')).toString()
const TAIL = fs.readFileSync(path.join(Config.WWW, 'tail.html')).toString()

const formats = {
	'.html': 'text/html',
	'.css': 'text/css',
	'.js': 'text/javascript'
}

db.Init()

const server = createServer(async (req, res) => {
	function sendText(text) {
		res.setHeader('Content-Type', 'text/plain; charset=utf-8')
		res.end(text)
	}
	function sendFile(file) {
		fs.readFile(file, {}, (err, data) => {
			if (err) {
				sendText('Не удалось отправить Вам файл :(')
			}
			else {
				let ContentType = formats[path.extname(file)]
				if (!ContentType)
					ContentType = 'text/plain'
				res.setHeader('Content-Type', ContentType)
				res.end(data)
			}
		})
	}
	function notFound() {
		res.statusCode = 404
		fs.readFile(path.join(Config.WWW, '404.html'), (err, data) => {
			if (err)
				sendText('Файл не найден!')
			else {
				res.end(data)
			}
		})
	}
	try {
		const url = new URL(`http://${req.headers.host}${req.url}`)
		const pathname = url.pathname

		const tokens = pathname.split('/').filter(el => el != '')
		const People = db.getPeople()
		const Verse = db.getVerse()

		if (tokens.length == 0)
			sendFile(path.join(Config.WWW, '/index.html'))
		else {
			const command = tokens[0]
			switch (command) {
				case 'api':
					if (tokens.length == 1) {
						sendText('API сайта kompoet.ru')
						break
					}
					else {
						let body = ''
						req.on('data', chunk => {
							body += chunk.toString()
						})
						req.on('end', () => {
							let index
							let data
							body = querystring.parse(body)
							switch (tokens[1]) {
								case 'people':
									index = Number(tokens[2])
									if (isNaN(index)) {
										data = People
										res.setHeader('Content-Type', 'application/json')
										res.end(JSON.stringify(data))
									}
									else {
										if (People[index]) {
											data = People[index]
											res.setHeader('Content-Type', 'application/json')
											res.end(JSON.stringify(data))
										}
										else {
											res.statusCode = 404
											res.sendText('Нет человека с таким индексом!')
										}
									}
									break
								case 'versepart':
									data = {}
									index = Number(tokens[2])
									if (isNaN(index))
										index = 0
									if (index < 0) {
										res.statusCode = 404
										sendText('Отрицательный индекс!')
										break
									}
									let countVerse = db.getVerseCount()
									let countBlock 
									let ostatok = countVerse % 20
									if (ostatok == 0)
										countBlock = countVerse / 20
									else
										countBlock = (countVerse - ostatok) / 20 + 1
									if (index > countBlock - 1) {
										res.statusCode = 404
										sendText('Слишком большой индекс! Стихи закончились')
									}
									else {
										let maxIndex = countVerse - index * 20
										if (maxIndex >= 20)
											for (let i = maxIndex; i > maxIndex - 20; --i)
												data[i] = Verse[i]
										else 
											for (let i = maxIndex; i > 0; --i)
												data[i] = Verse[i]
										res.setHeader('Content-Type', 'application/json')
										res.end(JSON.stringify(data))
									}
									break
								case 'pre':
									index = tokens[2]
									if (Verse[index]) {
										db.getPreview(tokens[2]).then(text => sendText(text))
									}
									else {
										res.statusCode = 404
										sendText('Не найдено!')
									}
									break
								case 'checklogin':
									if (db.peopleByLogin(tokens[2]))
										sendText('Логин занят')
									else {
										res.statusCode = 404
										sendText('Логин свободен')
									}
									break
								case 'checkpassword':
									if (db.checkPassword(body.login, body.password))
										sendText('Пароль верный')
									else {
										res.statusCode = 403
										sendText('Пароль неверный')
									}
									break
								case 'signup':
									db.addPeople(body.name, body.subname, body.login, body.password)
									res.writeHead(301, {location: '/login'})
									res.end()
									break
								case 'addverse':
									if (db.checkPassword(body.login, body.password)) {
										db.addVerse(body.title, body.login, body.genre, body.text)
										res.writeHead(301, {location: '/'})
										res.end()
									}
									else {
										res.statusCode = 403
										res.sendText('Нет прав! Перезайдите')
									}
									break
								default:
									res.statusCode = 404
									res.sendText(`API не имеет команды '${command}'`)
							}
						})
					}
					break
				case 'last':
					res.writeHead(301, {location: `${db.getVerseCount()}`})
					res.end()
					break
				case 'random':
					res.writeHead(301, {location: `${Math.floor(Math.random() * (db.getVerseCount() - 1)) + 1}`})
					res.end()
					break
				case 'signup':
					if (tokens.length == 1)
						sendFile(path.join(Config.WWW, 'signup.html'))
					else
						notFound()
					break
				case 'login':
					if (tokens.length == 1)
						sendFile(path.join(Config.WWW, 'login.html'))
					else
						notFound()
					break
				case 'add':
					if (tokens.length == 1)
						sendFile(path.join(Config.WWW, 'add.html'))
					else
						notFound()
					break
				default:
					if (db.peopleByLogin(command)) {
						sendText('Есть такой пользователь!')
					}
					else if (/^\d+$/.test(command)) {
						const v = Verse[command]
						if (v) {
							const author = People[v.AUTHOR]
							const fullName = `${author.NAME} ${author.SUBNAME}`
							let ret = HEAD.replace('${TITLE}', `${v.TITLE} | ${fullName}`)
							ret += `<div id="fullverse"><h1>${v.TITLE}</h1><a href="${author.LOGIN}"><i>${fullName}</i></a>`
							fs.readFile(path.join(Config.DB, 'verse', command, 'text'), 'utf8', (err, data) => {
								if (!err) {
									ret += `<pre>${data}</pre>${TAIL}</div>`
									res.end(ret)
								}
							})
						}
						else
							notFound()
						break
					}
					const filePath = path.join(Config.WWW, pathname)
					fs.exists(filePath, (e) => {
						if (e)
							sendFile(filePath)
						else
							notFound()
					})
			}
		}
	} catch (err) {
		res.statusCode = 500
		sendText('Server Error')
	}
})

server.listen(Config.PORT, console.log('Сервер запущен!'))
